// Demo app for sending SMS messages to a scrolling LED.
// Code based on http://code.google.com/p/microbridge/
// (c) 2011, Curious Inventor, LLC  
// Feel free to share, but please give attribution to CuriousInventor.com
// support@CuriousInventor.com
// Full tutorial at http://curiousinventor.com/sms-to-scrolling-led

#include <SPI.h>
#include <Adb.h>




/*
http://glowproducts.com/batteryoperated/ledbeltbuckle/

There are three buttons, on/off, page up, and page down.  You press these to program in a message.
Or, in our case, call pulse_pin(UP_DOWN_PIN) etc to simulate a short button press. That function acts
like the switch, shorting one of its sides to ground, and then leaving it open again by changing the pin
into an INPUT, which is a high impedance, or open circuit.

Programming sequence:

Turn on by pressing the on/ off button.  
Turn off by pressing the on/ off button--depends on mode.

The on/off button also acts as an ENTER button, and a space while entering in text.

After turn on, pressing UP will start cycling through the main menu options, press enter to select mode.
  -SWITCH: Change which memory bank is displayed. 6 banks available, 512 chars in the first bank, 
          256 on the rest.  Press enter again to select.
  -INPUT: Change the message at the currently selected bank. See below.
  -SPEED: 1-9.  Scroll to the desired number, press enter.
  -LUM: 1-9
  
  
  More details on the INPUT mode:
 
   Once in character input mode, use the UP and DOWN buttons to cycle thru the character array. 
   AFTER pressing one of the UP or DOWN buttons, ENTER confirms that character, and moves on to 
   the next character. 
   
   Note that pressing ENTER before pressing either UP or DOWN will enter a space.
   
   So, to enter an 'A', you would first need to press UP and then DOWN, and then ENTER, otherwise
   you'll enter a space to start off with.
  
  Press DOWN for 3s to save the message, or just wait 10s.
  
  Hold ENTER for 3s to delete a character.
  
  Character array: 
  ascii table starting at 'A' (65 decimal), going to 'z' (122), 
  then starting at '!' (33), going to '@' (64),
  then 4 weird characters and back to 'A'
  
  
  
  We make a lookup function to find out the number of presses for a given number, get_num_presses(unsigned char).
  
  




*/


#define ON_OFF_PIN 2
#define UP_PIN 3
#define DOWN_PIN 4
#define PULSE_DUR 30 //20 // note: these delay times seem to be effected by battery level.
#define DEBUG_PRINT 0
int PAUSE_DUR  = 85;

// Adb connection.
Connection * connection;

// Elapsed time for ADC sampling
long lastTime;
long slowTimer;
long pulseStartTime;
long pauseStartTime;

char on_off_mode = 0;
char up_mode = 0;
char down_mode = 0;

char pulseActive = 0;
char pausing = 0;

unsigned char cur_letter_pos;
unsigned char target_letter_pos;

void ground_pin(unsigned char pin, unsigned char grounded);
void reset_pulse(void);
void pulse_pin(unsigned char pin);
void check_pause(void);
unsigned char get_num_presses(unsigned char);
void check_for_sms(void);

enum DISPLAY_COMM_STATE {
  DO_NOTHING = 0,
  ENTER_MENU,
  SELECT_INPUT,
  ENTER_MSG_CHANGE,
  START_SCROLLING_TO_LETTER,
  SCROLLING
} display_state;



unsigned char buffer[256];
unsigned char sms_buffer[256];
int sms_buf_index;
int write_buf_index = 0;
int read_buf_index = 0;
char sms_ready_flag = 0;

// Event handler for the shell connection. 
void adbEventHandler(Connection * connection, adb_eventType event, uint16_t length, uint8_t * data)
{
  int i;

  // Data packets contain two bytes, one for each servo, in the range of [0..180]
  if (event == ADB_CONNECTION_RECEIVE)
  {
    //servos[0].write(data[0]);
    //servos[1].write(data[1]);
    sms_buf_index = 0;
    //Serial.print("SMS Received: ");
    for (i=0;i<length;i++) {
      //Serial.print(data[i]);
      sms_buffer[sms_buf_index++] = data[i];      
    }
    sms_ready_flag = 1;
  
  }

}

void check_for_sms() {
  int i;
  if (sms_ready_flag && display_state == DO_NOTHING) {
    write_buf_index = 0; // just in case
    while(write_buf_index < sms_buf_index) {
       buffer[write_buf_index] = sms_buffer[write_buf_index];
       //Serial.print((int)write_buf_index);Serial.print(": ");Serial.println((int)(buffer[write_buf_index]));
       write_buf_index++;
    }
    buffer[write_buf_index] = 0; // NULL terminate the string for printing.
    Serial.print("SMS Recevied: ");Serial.println((char *)buffer);
    display_state = ENTER_MENU;    
    sms_ready_flag = 0;    
  }
}

void setup()
{

  // Initialise serial port
  Serial.begin(57600);
  
  pinMode(ON_OFF_PIN,INPUT);
  pinMode(UP_PIN,INPUT);
  pinMode(DOWN_PIN,INPUT);
  digitalWrite(ON_OFF_PIN,0);
  digitalWrite(UP_PIN,0);
  digitalWrite(DOWN_PIN,0);

  // Note start time
  lastTime = 0;
  slowTimer = 0;
  pulseStartTime = 0;
  pauseStartTime = 0;
  
  display_state = DO_NOTHING;

  // Initialise the ADB subsystem.  
  ADB::init();

  // Open an ADB stream to the phone's shell. Auto-reconnect
  connection = ADB::addConnection("tcp:4567", true, adbEventHandler);  
}

void loop()
{
   int num_presses;
//  if ((millis() - lastTime) > 20)
//  {
//    uint16_t data = analogRead(A0);
//    connection->write(2, (uint8_t*)&data);
//    lastTime = millis();
//  }

// To test out the scrolling LED writing, enter a phrase into the Serial Monitor, and make sure to turn on 'Newline' in 
// option dropdown next to the baud rate.
  if (Serial.available()) {
    if (display_state == DO_NOTHING) { // don't accept any new characters while writing out something.
      buffer[write_buf_index] = Serial.read();
      if (buffer[write_buf_index++] == '\n') { // new line, process
         
         buffer[--write_buf_index] = 0; // cap the string for printing
         Serial.print("Process phrase: ");Serial.println((char *)buffer);
         
         display_state = ENTER_MENU;  // uses the write_buf_index as the total length.
         
  //       while (read_buf_index < write_buf_index - 1) {
  //          num_presses = get_num_presses(buffer[read_buf_index++]);
  //          Serial.println(num_presses);
  //         
  //       }
  //       write_buf_index = 0;
      }
    }
    
  }
  
  check_for_sms();
  
  continue_processing_display_comm();
  reset_pulse();
  check_pause();
  // Poll the ADB subsystem.
  ADB::poll();
}



void continue_processing_display_comm() {
  unsigned char current_letter;
  if (!pausing && !pulseActive) {
    switch (display_state) {
      case DO_NOTHING:
      
        break;      
      case ENTER_MENU: // kick off setting a new message on the display. We assume we're in the default msg display screen.
        cur_letter_pos = 0; // We always start off at letter 'A', or pos 0.
        read_buf_index = 0;
        PAUSE_DUR = 90;
        pulse_pin(UP_PIN); // this goes into mode-switching state
        display_state = SELECT_INPUT;
        
        break;
      case SELECT_INPUT: 
        pulse_pin(UP_PIN); // keep going to the INPUT mode-selection 
        display_state = ENTER_MSG_CHANGE;
        break;
      case ENTER_MSG_CHANGE:
        pulse_pin(ON_OFF_PIN); // select INPUT mode.
        display_state = START_SCROLLING_TO_LETTER;
        break;
      case START_SCROLLING_TO_LETTER: // continue writing letters until we've written out the entire buffer
        PAUSE_DUR = 90;
        if (read_buf_index < write_buf_index) {
          current_letter = buffer[read_buf_index++];
          if (DEBUG_PRINT) {
            Serial.print("cur letter: ");Serial.print((char)current_letter);Serial.print(" cur pos: ");Serial.println((int)cur_letter_pos);
          }
          if (current_letter == ' ') { // special case, we need to move off the letter, and back on.
            pulse_pin(ON_OFF_PIN);   
           display_state = START_SCROLLING_TO_LETTER; // this immediately moves us to the next letter.         
          }
          else {
            PAUSE_DUR = 15;
            target_letter_pos = get_num_presses(current_letter);
            if (target_letter_pos == 150) { // bail, don't recognize
               display_state = START_SCROLLING_TO_LETTER;
               return;
            }
            
            if (DEBUG_PRINT) {
              Serial.print("target pos: ");Serial.println((int)target_letter_pos);
            }
            if (target_letter_pos >= cur_letter_pos) { // even if we're starting at the right pos, we need to move
                                                       // off and back on to the letter to avoid entering a space.
              pulse_pin(UP_PIN);
              cur_letter_pos++;
            }
            else {
              pulse_pin(DOWN_PIN);
              cur_letter_pos--;
            }
            display_state = SCROLLING;
          }          
          
        }
        else { // we're done writing out the letters, issue the finished command by holding DOWN for 3s
          write_buf_index = 0;
          Serial.println("Finished writing, exiting INPUT mode");
          pinMode(DOWN_PIN,OUTPUT);
          delay(3100);
          pinMode(DOWN_PIN,INPUT);
          display_state = DO_NOTHING;
        }
        break;
      case SCROLLING:
        if (DEBUG_PRINT) {
          Serial.print("cur_pos: ");Serial.println((int)cur_letter_pos);
        }
        if (cur_letter_pos == target_letter_pos) { // we've scrolled to the correct letter, now enter it in.
          PAUSE_DUR = 90;
          pulse_pin(ON_OFF_PIN); // enter in this selection
          //delay(500); // long delay to store number
          display_state = START_SCROLLING_TO_LETTER;
        }
        else if (cur_letter_pos < target_letter_pos) {
           pulse_pin(UP_PIN);
           cur_letter_pos++;
           display_state = SCROLLING;
        }
        else {
          pulse_pin(DOWN_PIN);
          cur_letter_pos--;
          display_state = SCROLLING; 
        }
        break;
      
    }
    
   
    
  }  
}

void ground_pin(unsigned char pin, unsigned char grounded) {
  if (grounded == 1) {    
    pinMode(pin,OUTPUT);
  }
  else
    pinMode(pin,INPUT);
}

void pulse_pin(unsigned char pin) {
  pinMode(pin,OUTPUT);
  //digitalWrite(13,1);
  pulseActive = 1;
  pulseStartTime = millis();
  if (DEBUG_PRINT) {
    Serial.print("pulse: ");Serial.println((int)pin);
  }

}

// this is called every loop to release the last pulse.  Setting the pinMode to input effectively 
// releases a button.
void reset_pulse() {
  if (pulseActive && (millis() - pulseStartTime > PULSE_DUR)) {
    pinMode( ON_OFF_PIN,INPUT);
    pinMode(UP_PIN,INPUT);
    pinMode(DOWN_PIN,INPUT);
   // digitalWrite(13,0);
    pulseActive = 0;
    // after each pulse, we pause for a bit
    pausing = 1;
    pauseStartTime = millis();
  }
}

// this takes a target character in, and looks up how many presses from A it will take to get to that letter.
// Or, in other words, the address of that letter.

//  ascii table starting at 'A' (65 decimal), going to 'z' (122), 
//  then starting at '!' (33), going to '@' (64),
//  then 4 weird characters and back to 'A'
unsigned char get_num_presses(unsigned char letter) {
  
//  'A' to 'z'
  if (letter >= 'A' && letter <= 'z') {
     return letter - 'A';
  }
  else if (letter >= '!' && letter <= '@') { // '!' to '@'
    return ('z' - 'A') + 1 + letter - '!';
  }
  else 
    return 150; // some heart thing?
  
}

void check_pause() {
  if (pausing && millis() - pauseStartTime > PAUSE_DUR) {
     pausing = 0; 
  }
}

